/*
  Fichero Ejemplo1.java
	- Un fichero fuente (*.java) puede contener ms de una clase, pero slo una puede ser public
	- El nombre del fichero debe coincidir con el de la clase public (y extensin .java)
	- Una clase puede ser public o package (este ltimo, por defecto)
	- Todas las clases java se agrupan en packages (libreras de clases). Si las clases no se definen 
	como pertenecientes a un paquete, se utliza un paquete por defecto (default package) que coincide
	con el directorio actual
*/

import java.util.ArrayList;
import java.awt.*;
/* 
 Paquetes (agrupacin de clases) que utiliza la aplicacin
 - El paquete java.lang se importa siempre automticamente
 - La declaracin de los paquetes utilizados debe hacerse antes de la definicin de cualquier clase
*/

class Ejemplo1 {
/* Java es un lenguaje orientado a objetos puro: todo son clases (no puede definirse una variable o una 
 * funcin que no pertenezca a una clase
 * - Las clases son plantillas para la creacin de objetos (o instancias) y estn formadas por variables
 * miembro (definen el estado del objeto) y funciones miembro o mtodos (definen el comportamiento del 
 * objeto) 
 */
	
	
	/* La clase "Ejemplo1" slo define el mtodo "main()", que es el programa principal de una aplicacin Java.
	 * - Tiene como misin iniciar y finalizar la ejecucin de la aplicacin
	 * - Debe ser empleado para crear un conjunto de objetos que se comuniquen entre s mediante mensajes
	 * (invocaciones a mtodos) con el fin de obtener el resultado final 
	 * - La funcin "main()" se declara como:
	 *		- "public", puede ser utilizada por cualquier clase
	 *		- "static", es un mtodo de clase, es decir, puede ser utilizada la funcin main aunque no se haya
	 *		creado ningn objeto de la clase "Ejemplo1"
	 *		- no devuelve nada, indicado por el tipo de retorno "void" (al estilo de C)
	 *		- tiene como argumentos, un array de cadenas que contiene los argumentos de la lnea de ordenes
	 *		("String arg[]")
	 */ 
	public static void main(String args[]) {
		// Visualiza la cadena por pantalla, utilizando la variable miembro esttica "out" de la clase "System", definida
		// en el paquete "java.lang". "out" se corresponde con la salida estandar (el campo "in" es la entrada estndar
		// y el campo "err" es la salida de error estndar) 
		System.out.println("Comienza main()...");
		
		/* Se crea un objeto de la clase "Circulo" que se podr referenciar a travs de "c". Es equivalente a:
		 * Circulo c; //Definicin de una referencia a un objeto de la clase crculo (no el propio objeto)
		 * c= new Circulo(2.0, 2.0, 4.0) // Se crea con "new" el objeto en s mismo y su referencia se guarda en c
		 * 
		 * Nota:
		 *	- Todos los objetos son dinmicos, se crean en el montn con la instruccin "new"
		 *	- En Java no hay punteros, pero las referencias a objetos juegan un papel anlogo, ya
		 *	que permiten acceder al objeto creado
		 *	- En Java no hay que preocuparse por liberar memoria de los objetos que estn en desuso,
		 *	lo hace automticamente el recolector de basura (garbage collector)
		 * ESTILO DE PROGRAMACIN:
		 *		- Para nombrar a las clases se utilizan identificadores con letras en maysculas
		 *		- Para nombrar a los objetos se utilizan identificadores con letras en minsculas
		 */
		Circulo c = new Circulo(2.0, 2.0, 4.0);
		System.out.println("Radio = " + c.r + " unidades");
		System.out.println("Centro = (" + c.x + ", " + c.y + ") unidades");
		
		Circulo c1= new Circulo(1.0, 1.0, 2.0);
		Circulo c2= new Circulo(0.0, 0.0, 3.0);
		// Se comparan el radio del circulo c1 con el radio del circulo c2, el mayor se referencia por c
		// Para ello se utiliza un mtodo de instancia (de c1, en concreto) que devuelve una referencia al
		// circulo de radio mayor
		c = c1.elMayor(c2);
		System.out.println("El mayor radio es " + c.r + ".");
		
		// Se hace la misma comparacin, pero ahora utilizando un mtodo de clase (definido como static)
		// - El mtodo elMayor est sobrecargado porque, an teniendo el mismo nombre, tienen diferente cdigo
		c= new Circulo(); // por defecto, c.r = 0.0 y (c.x, c.y) = (0, 0). Ver constructor de clase
		c= Circulo.elMayor(c1, c2);
		System.out.println("El mayor radio es " + c.r + ".");
		
		
		
		VentanaCerrable ventana= new VentanaCerrable("Ventana abierta al mundo...");
										// clase derivada de la clase "Frame", definida en el paquete "java.awt", para
										// superar alguna limitacin de la clase "Frame" (ver VentanaCerrable.java)
										
		ArrayList v= new ArrayList();	// clase definida en "java.util" y permite almacenar referencias a objetos de 
										// distintas clases
		
		CirculoGrafico cg1= new CirculoGrafico(200, 200, 100, Color.red);
		CirculoGrafico cg2= new CirculoGrafico(300, 200, 100, Color.blue);
		RectanguloGrafico rg= new RectanguloGrafico(50, 50, 450, 350, Color.green);
		
		v.add(cg1);
		v.add(cg2);
		v.add(rg);
		
		PanelDibujo mipanel= new PanelDibujo(v); 	// Crea un objeto de la clase "PanelDibujo" sobre el que se hacen los
													// preparativos para visualizar en una ventana los objetos dibujables
													// que se quiera, previamente, almacenados (en concreto una referencia
													// a ellos) en la "ArrayList" v
		
		ventana.add(mipanel);						// Volcado del panel sobre la ventana y preparativos para visualizar
		ventana.setSize(500, 400);					// su contenido
		ventana.setVisible(true);
		
		System.out.println("Termina main()...");
		
		
	} // fin de main()
} // fin de la definicion de la clase Ejemplo1
