//Fichero PanelDibujo.java

import java.awt.*;
import java.util.ArrayList;
import java.util.Iterator;

/*
 * La clase "PanelDibujo" deriva de la clase "Panel" definida en el paquete "java.awt". La clase "Panel" deriva
 * de la clase "Container", que deriva de "Component", que deriva de "Object".
 *	- La clase "Object" es la clase ms general de Java, es la base de la jerarqua de objetos
 *	- La clase "Component" se refiere a todos los objetos de Java que tienen representacin grfica (botones, 
 *	barras de desplazamiento, etc.)
 *	- La clase "Container" son objetos grficos del AWT, capaces de contener otros objetos del AWT
 *		El AWT (Abstract Windows Toolkit) es la librera de clases de Java que permite crear interfaces grficas de usuario
 *	- La clase "Panel" define uno de los "Container" ms sencillos, capaces de contener otros elementos grficos (como
 *	paneles) y sobre los que se puede dibujar
 */

public class PanelDibujo extends Panel {	// "PanelDibujo" hereda de "Container" y "Component" las capacidades grficas
											// a travs de "Panel"
	//Utiliza una variable miembro: una lista con las referencias de los objetos a dibujar
	private ArrayList v;
	
	// Definicin del constructor de la clase
	public PanelDibujo(ArrayList va) {
		super(new FlowLayout());    // se inicializa el objeto "PanelDibujo" con el constructor de la superclase,
									// al que se pasa un objeto de la clase "FlowLayout" (creado en la invocacin)
									// La clase "FlowLayout" se ocupa de distribuir de una determinada forma (de izq a der,
									// y de arriba abajo) los componentes grficos que se aaden a un Container como Panel
		this.v= va;
	}


	/* Nocin de POLIFORMISMO
	 * En Java, y en general, en los lenguajes OO, se utiliza el concepto de poliformismo. La idea bsica es que
	 * una refencia a un objeto de una determinada clase es capaz de servir de referencia o de nombre a objetos de cualquiera
	 * de sus clases derivadas. Por ejemplo, en Java se puede hacer lo siguiente:
	 *		Geometria geom1, geom2;
	 *		geom1= new RectanguloGrafico(0.0, 0.0, 200.0, 100.0, Color.red);
	 *		geom2= new CirculoGrafico(200.0, 200.0, 100.0, Color.blue);
	 * Es decir, se crean referencias de la clase "Geometria" que posteriormente apuntan a objetos de las clases derivadas,
	 * "RectanguloGrafico" y "CirculoGrafico", respectivamente.
	 * Existe una limitacin: ya que son referencias a la clase "Geometria" slo pueden hacer uso de los mtodos definidos 
	 * en dicha clase, es decir, de los mtodos "perimetro()" y "area()"
	 *
	 * Lo mismo se puede hacer con las interfaces, de modo que es posible utilizar una refencia del tipo correspondiente
	 * a una interfaz para manejar objetos de clases que implementan dicha interfaz. Se puede definir:
	 *		Dibujable dib1, dib2;
	 *		dib1= new RectanguloGrafico(0.0, 0.0, 200.0, 100.0, Color.red);
	 *		dib2= new CirculoGrafico(200.0, 200.0, 100.0, Color.blue);
	 * Como en el claso de las clases, slo pueden ser utilizados con los mtodos definidos en la interfaz "Dibujable"
	 *
	 * El poder utilizar nombres de una super-clase o de una interfaz permite tratar de un modo unificado objetos distintos,
	 * aunque pertenecientes a distintas sub-clases o bien a clases que implementan dicha interfaz. Esta es la idea en la
	 * que se basa el POLIFORMISMO
	 */

	// Redefinicin del mtodo "paint()"
	/* El mtodo "paint()" es un mtodo heredado de "Container", que a su vez redefine el mtodo heredado de "Component".
	 * Este mtodo tiene la peculiaridad de que el programador no debe invocarlo, se encargan de ello Java y el SO. En 
	 * sentido, el programador debe preparar la ventan y el panel en el que va a dibujar, as como redefinir el mtodo 
	 * "paint()" para definir las operaciones grficas que quiere realizar. El SO y Java llaman automticamente a "paint()"
	 * cada vez que entienden que la ventana debe ser dibujada o redibujada
	 */
	public void paint (Graphics g) {
		Dibujable dib; 	// se define una referencia del tipo correspondiente a la interfaz "Dibujable", con lo que 
						// se podr utilizar para referirse a cualquier objeto que implemente dicha interfaz
		Iterator it;	// se define una referencia del tipo correspondiente a la interfaz "Iterator" definida en 
						// el paquete "java.util". Esta interfaz proporciona los mtodos:
						//		-hasNext() que chequea si la coleccin de elementos que se est recorriendo tiene ms eltos.
						//		-next() que devuelve el siguiente elemento de la coleccin
		
		
		it= v.iterator();	// La clase ArrayList de Java, es una coleecin de datos, que implenta la interfaz "Iterator",
							// por lo que es posible obtener una referencia "Iterator" de los elementos de la lista v, 
							// utilizando el mtodo "iterator()" de la clase "ArrayList"
		while (it.hasNext()) {
			dib = (Dibujable)it.next(); // "it.next()" devuelve una refencia al siguiente objeto de la lista accesible
										// a travs del iterador "it". Dado que puede ser cualquier tipo de objeto (Object),
										// es necesario forzar su conversin de tipo de modo que sea compatible con la
										// interfaz "Dibujable"
			dib.dibujar(g);		// se invoca al mtodo que dibuja realmente el objeto
		}
	}
}