// Fichero VentanaCerrable.java
import java.awt.*;
import java.awt.event.*;



/*
 * La clase "VentanaCerrable" se deriva de la clase "Frame" para solventar alguna deficiencia de esta ltima. En particular
 * la clase "Frame" de Java no responde a las acciones normales en Windows para cerrar una ventana o aplicacin. La
 * subclase se caracteriza por implementar la interfaz "WindowListener" que especifica las funciones que debe implementar
 * un objeto capaz de controlar los eventos que recibe una ventana (conviene aclarar que "Frame" se deriva de "Window"
 */
class VentanaCerrable extends Frame implements WindowListener {

	// Definicin del constructor genrico
	public VentanaCerrable() {
		super();
		// Intencionadamente no se establece un manejador de eventos de ventana, por lo que si en el mtodo principal
		// del fichero Ejemplo1.java se sustituye la creacin del objeto ventana por el constructor sin argumentos,
		// se observarn las deficiencias de la clase "Frame"
	}

	// Defincin del constructor particular
	public VentanaCerrable(String title) {
		super(title);
		setSize(500, 500);
		addWindowListener(this); // se utiliza el mtodo addWindowListener de la clase Frame para definir
								 // que ste es el objeto manejador de los eventos sobre ventanas
	}

	// Mtodos de la interfaz WindowsListener, en principio, todos vacos salvo el mtodo que trata
	// el evento cerrar ventana
	public void windowActivated(WindowEvent e) {;}
	public void windowClosed(WindowEvent e) {;}
	public void windowClosing(WindowEvent e) {System.exit(0);}
	public void windowDeactivated(WindowEvent e) {;}
	public void windowDeiconified(WindowEvent e) {;}
	public void windowIconified(WindowEvent e) {;}
	public void windowOpened(WindowEvent e) {;}

}

